;------------------------------------------------------------------------------
;
;  GenPIC
;
;  - Generic PIC utility PCB
;  - program structure is a main loop calling the various handlers, all
;  handlers must be non-blocking to continue program operation
;  - state handles the needed operator modes and actions
;  - Written for the PIC18F66K80 microcontroller and GenPIC PCB
;
;  11Feb2017 v0.1 A. Cooper
;  - initial version
;

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"
	include	"config.asm"
	
;------------------------------------------------------------------------------
;  External Resources
	extern	UserInit, UserLEDCheck, UserPBCheck, UserEncCheck
	extern	userFlag, userEncA, userEncB
	extern	TimeInit, TimeKeep, timeMil
	extern	DispInit
	extern	StateInit, StateHandle
	extern	SerInit, SerGetCh
	extern	ParseCmd, CmdCheck
	extern	HostMsgRev, HostMsgRst, HostLine, HostPrompt
	extern	ADInit

;------------------------------------------------------------------------------
;  Reset
ResetVector	org	0x0000
	goto	Init

;------------------------------------------------------------------------------
;  Interrupt Routines
;  - no effort is made to separate high or low priority interrupts
;  - interrupts serviced
;    TMR2 :increment timeCent and clear
;    EUSART1  :move character into buffer
;
ISRH	org	0x0008
	goto	ISR
ISRL	org	0x0018
	goto	ISR
ISR	btfss	PIR1,TMR2IF,A ;check for timer 2 interrupt
               goto	ISRSer
	bcf	PIR1,TMR2IF,A
	incf	timeMil,F,A ;increment time base
	bsf	STATUS,C	;sample encoder values at 1kHz
	btfss	userRead,userUIO0,A
	 bcf	STATUS,C
	rlcf	userEncB,F,A
	bsf	STATUS,C
	btfss	userRead,userUIO1,A
	 bcf	STATUS,C
	rlcf	userEncA,F,A
ISRSer	btfsc	PIR1,RC1IF,A ;check for serial receive
	 call	SerGetCh
ISREnd	retfie	FAST

;------------------------------------------------------------------------------
;  module code
GenPIC	code

;------------------------------------------------------------------------------
;  Init
;  - set up all needed registers and peripherals after reset
;
Init	;hardware init
	movlw	0x20	;disable ECAN module
	movwf	CANSTAT,A
	clrf	RCSTA2,A	;disable EUSART2
	;module setups
	call	TimeInit	;set up timer module
	call	DispInit	;set up diplay module
	call	UserInit	;initialize user interface module
	call	StateInit	;init state handler module
	call	SerInit	;init serial port
	call	ADInit	;init AtoD converter
	;general interrupt setup
	bcf	RCON,IPEN,A	;no interrupt priorities
	bsf	INTCON,GIE,A	;ensure global interrupts enabled
	bsf	INTCON,PEIE,A ;enable peripheral interrupts
	;show we are alive
	bsf	userFlag,userLED1On
	bsf	userFlag,userLED2On
	call	UserLEDCheck
	call	StateHandle	;init state 0 and display splash
	call	HostMsgRev	;initial serial ouput
	call	HostMsgRst
	call	HostLine
	call	HostPrompt

;------------------------------------------------------------------------------
;  Main
;
Main	clrwdt		;clear watchdog timer
	call	TimeKeep	;keep clock running
	call	UserLEDCheck	;update LED's as per userFlag
	call	UserPBCheck	;check pushbuttons
	call	UserEncCheck	;check encoder
	call	StateHandle	;handle state controller
	call	CmdCheck	;check for serial commands
	goto	Main
	
	end
;- end GenPIC.asm -------------------------------------------------------------
