;------------------------------------------------------------------------------
;  analog.asm
;
;  - handle the on-board 12-bit AtoD converter
;
;  16Mar2016 A. Cooper
;  - initial version
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
channel	res	1	;temp channel register

;------------------------------------------------------------------------------
;  Available Resources
	global	ADRead, ADChan, ADInit

;------------------------------------------------------------------------------
;  AtoD set constants
adSet0	equ	b'00000001'	;set AtoD converter on, set channel AN0
adSet1	equ	b'00000000'	;Vref+ = AVdd, Vref- = AVss
adSet2	equ	b'10010100'	;Fosc/4 and Tad=4
adChan0	equ	b'00011111'	;AN0 through AN4 only
adChan1	equ	b'00000000'

;------------------------------------------------------------------------------
;  Module Code
analog	code

;------------------------------------------------------------------------------
;  ADInit
;  - set up the on-board AtoD converter
;
ADInit	movlw	adSet0	;Send setup 0 (see above)
	movwf	ADCON0,A
	movlw	adSet1	;Send setup 1 (see above)
	movwf	ADCON1,A
	movlw	adSet2	;Send setup 2 (see above)
	movwf	ADCON2,A
	banksel	ANCON0	;set analog input channels
	movlw	adChan0
	movwf	ANCON0
	movlw	adChan1
	movwf	ANCON1
	return


;------------------------------------------------------------------------------
;  ADChan
;  - set the A/D to channel given in W
;
ADChan	andlw	0x1F	;mask channel bits
	movwf	channel,A
	rlncf	channel,F,A
	rlncf	channel,W,A
	iorlw	adSet0	;or with setup bits
	movwf	ADCON0,A	;set channel	
	return

;------------------------------------------------------------------------------
;  ADRead
;  - read the channel
;  - results in ADRESH and ADRESL 
;
ADRead	bsf	ADCON0,GO,A	;start a conversion
ADReadWait	btfsc	ADCON0,GO,A	;wait for conversion complete
	 goto	ADReadWait
	return

	end
;-- end analog.asm -------------------------------------------------------------
