;------------------------------------------------------------------------------
;  command.asm
;  - handle serial commands
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2007-2015 all rights reserved
;
;  15Aug2007 A. Cooper
;  - initial version
;  29Jan2012 A. Cooper
;  - modified for iSource
;  03Jan2014 A. Cooper
;  - modified for LensCtrl
;  30May2015 A. Cooper
;  - rewritten for PIC18F
;  - modified for Incline
;  - changed to relocatable code
;  6Feb2017 A. Cooper
;  - modified for GenPIC
;
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;  Global Definitions
	include	PIC18F66K80.inc ;processor specific definitions
	include	"defs.asm"

;------------------------------------------------------------------------------
;  Declarations
	UDATA_ACS
cmdCode	res	1
	
;------------------------------------------------------------------------------
;  Available Resources
	global	CmdCheck, CmdExecute
	global	cmdCode
	
;------------------------------------------------------------------------------
;  External Resources
	extern	HostMsgHelp, HostLine, HostHex, HostLDec
	extern	HostDec2, HostHexSp, HostPrompt
	extern	resHi, resLo
	extern	serFlag, SerSend, SerClear
	extern	ParseCmd, ParseDec, parsePrm
	extern	timeHour, timeMin, timeSec
	extern	encVal
	extern	ADRead, ADChan, TempConvert
	
;------------------------------------------------------------------------------
;  Module Code
command	code

;------------------------------------------------------------------------------
;  CmdCheck
;
;  - check for and parse serial commands
;  - issue a new serial prompt if needed
;
CmdCheck	btfss	serFlag,serCmd,A ;check for command ready
	 return
	call	ParseCmd	;parse the command in the serial buffer
	btfsc	serFlag,1,A	;still a good command?
	 call	CmdExecute	;execute the command
	call	SerClear	;clear the serial port buffer and pointers
	call	HostPrompt	;replace the prompt
	return
	
;------------------------------------------------------------------------------
;  CmdExecute
;
;  - excute a parsed command
;  - currently limited to 16 commands, can be changed by extending table
;  and changing mask
;
CmdExecute	movlw	0x0F	;ensure command<16
	andwf	cmdCode,F,A
	incf	cmdCode,F,A	;offset and x4 for PC words and GOTO
	rlncf	cmdCode,F,A
	rlncf	cmdCode,F,A
	movlw	UPPER CmdJmp ;do a computed goto
	movwf	PCLATU,A
	movlw	HIGH CmdJmp
	movwf	PCLATH,A
	movf	cmdCode,W,A
	addlw	LOW CmdJmp
	btfsc	STATUS,C,A
	 incf	PCLATH,F,A
CmdJmp	movwf	PCL,A
	nop
	goto	CmdReset	;00 full reset
	goto	CmdClock	;01 report clock
	goto	CmdEncoder	;02 set encoder value
	goto	CmdTemp	;03 report temperature
	goto	CmdSpare	;04
	goto	CmdSpare	;05
	goto	CmdSpare	;06
	goto	CmdSpare	;07
	goto	CmdSpare	;08
	goto	CmdSpare	;09 
	goto	CmdSpare	;0A
	goto	CmdSpare	;0B
	goto	CmdSpare	;0C
	goto	CmdSpare	;0D
	goto	CmdSpare	;0E
	goto	CmdHelp	;0F help to console
CmdSpare	return

;------------------------------------------------------------------------------
;  CmdReset
;
;  - reset the controller
;
CmdReset	reset

;------------------------------------------------------------------------------
;  CmdClock
;  - report the time to the console
;
CmdClock	movf	timeHour,W,A
	call	HostDec2
	movlw	':'
	call	SerSend
	movf	timeMin,W,A
	call	HostDec2
	movlw	':'
	call	SerSend
	movf	timeSec,W,A
	call	HostDec2
	call	HostLine
	return

;------------------------------------------------------------------------------
;  CmdEncoder
;  - set the encoder value as specified
;
CmdEncoder	call	ParseDec	;get new value from parameter
	movf	parsePrm+1,W,A
	movwf	encVal
	return

;------------------------------------------------------------------------------
;  CmdTemp
;  - report the temperature to the console
;
CmdTemp	movlw	0x04	;set A/D channel AN4 for thermistor
	call	ADChan
	call	ADRead
	call	ADRead	;read temperature
	movf	ADRESH,W,A	;get result
	movwf	resHi,A
	movf	ADRESL,W,A
	movwf	resLo,A
	call	TempConvert	;convert to decimal
	movlw	'-'	;output sign
	btfss	resHi,7,A
	 movlw	' '
	call	SerSend
	bcf	resHi,7,A
	swapf	resHi,W,A	;output value
	andlw	0x0F
	addlw	0x30
	call	SerSend
	movf	resHi,W,A
	andlw	0x0F
	addlw	0x30
	call	SerSend
	swapf	resLo,W,A
	andlw	0x0F
	addlw	0x30
	call	SerSend
	movlw	'.'
	call	SerSend
	movf	resLo,W,A
	andlw	0x0F
	addlw	0x30
	call	SerSend
	movlw	'C'
	call	SerSend
	call	HostLine
	return
	
;------------------------------------------------------------------------------
;  CmdHelp
;
;  - print command help
;
CmdHelp	call	HostMsgHelp	;send help text to serial console
	return

	end
;-- command.asm ---------------------------------------------------------------
